@@ -94,10 +94,15 @@ static const intel_x86_umask_t skl_br_misp_retired[]={
     .uflags = INTEL_X86_NCOMBO | INTEL_X86_PEBS | INTEL_X86_DFL,
   },
   { .uname = "NEAR_TAKEN",
-    .udesc  = "number of near branch instructions retired that were mispredicted and taken",
+    .udesc  = "Number of near branch instructions retired that were mispredicted and taken",
     .ucode  = 0x2000,
     .uflags = INTEL_X86_NCOMBO | INTEL_X86_PEBS,
   },
+  { .uname = "NEAR_CALL",
+    .udesc  = "Counts both taken and not taken retired mispredicted direct and indirect near calls, including both register and memory indirect.",
+    .ucode  = 0x200,
+    .uflags = INTEL_X86_NCOMBO | INTEL_X86_PEBS,
+  },
 };
 
 static const intel_x86_umask_t skl_cpu_clk_thread_unhalted[]={
@@ -129,6 +134,12 @@ static const intel_x86_umask_t skl_cpu_clk_thread_unhalted[]={
     .ucode  = 0x200,
     .uflags= INTEL_X86_NCOMBO,
   },
+  { .uname  = "RING0_TRANS",
+    .udesc  = "Counts when the current privilege level transitions from ring 1, 2 or 3 to ring 0 (kernel)",
+    .ucode  = 0x000 | INTEL_X86_MOD_EDGE | (1 << INTEL_X86_CMASK_BIT), /* edge=1 cnt=1 */
+    .uequiv = "THREAD_P:e:c=1",
+    .uflags= INTEL_X86_NCOMBO,
+  },
 };
 
 static const intel_x86_umask_t skl_cycle_activity[]={
@@ -219,10 +230,25 @@ static const intel_x86_umask_t skl_dtlb_load_misses[]={
     .uflags = INTEL_X86_NCOMBO,
   },
   { .uname = "WALK_COMPLETED",
-    .udesc  = "Misses in all TLB levels causes a page walk of any page size that completes",
+    .udesc  = "Number of misses in all TLB levels causing a page walk of any page size that completes",
     .ucode  = 0xe00,
     .uflags = INTEL_X86_NCOMBO,
   },
+  { .uname = "WALK_COMPLETED_4K",
+    .udesc  = "Number of misses in all TLB levels causing a page walk of 4KB page size that completes",
+    .ucode  = 0x200,
+    .uflags = INTEL_X86_NCOMBO,
+  },
+  { .uname = "WALK_COMPLETED_2M_4M",
+    .udesc  = "Number of misses in all TLB levels causing a page walk of 2MB/4MB page size that completes",
+    .ucode  = 0x400,
+    .uflags = INTEL_X86_NCOMBO,
+  },
+  { .uname = "WALK_COMPLETED_1G",
+    .udesc  = "Number of misses in all TLB levels causing a page walk of 1GB page size that completes",
+    .ucode  = 0x800,
+    .uflags = INTEL_X86_NCOMBO,
+  },
   { .uname = "WALK_ACTIVE",
     .udesc  = "Cycles with at least one hardware walker active for a load",
     .ucode  = 0x1000 | (0x1 <<  INTEL_X86_CMASK_BIT),
@@ -253,10 +279,25 @@ static const intel_x86_umask_t skl_itlb_misses[]={
     .uflags = INTEL_X86_NCOMBO,
   },
   { .uname = "WALK_COMPLETED",
-    .udesc  = "Misses in all TLB levels causes a page walk of any page size that completes",
+    .udesc  = "Number of misses in all TLB levels causing a page walk of any page size that completes",
     .ucode  = 0xe00,
     .uflags = INTEL_X86_NCOMBO,
   },
+  { .uname = "WALK_COMPLETED_4K",
+    .udesc  = "Number of misses in all TLB levels causing a page walk of 4KB page size that completes",
+    .ucode  = 0x200,
+    .uflags = INTEL_X86_NCOMBO,
+  },
+  { .uname = "WALK_COMPLETED_2M_4M",
+    .udesc  = "Number of misses in all TLB levels causing a page walk of 2MB/4MB page size that completes",
+    .ucode  = 0x400,
+    .uflags = INTEL_X86_NCOMBO,
+  },
+  { .uname = "WALK_COMPLETED_1G",
+    .udesc  = "Number of misses in all TLB levels causing a page walk of 1GB page size that completes",
+    .ucode  = 0x800,
+    .uflags = INTEL_X86_NCOMBO,
+  },
   { .uname = "WALK_DURATION",
     .udesc  = "Cycles when PMH is busy with page walks",
     .ucode  = 0x1000,
@@ -519,6 +560,14 @@ static const intel_x86_umask_t skl_l1d[]={
   },
 };
 
+static const intel_x86_umask_t skl_sq_misc[]={
+  { .uname = "SPLIT_LOCK",
+    .udesc  = "Number of split locks in the super queue (SQ)",
+    .ucode  = 0x1000,
+    .uflags = INTEL_X86_DFL,
+  },
+};
+
 static const intel_x86_umask_t skl_l1d_pend_miss[]={
   { .uname = "PENDING",
     .udesc  = "Cycles with L1D load misses outstanding",
@@ -578,17 +627,23 @@ static const intel_x86_umask_t skl_l2_lines_in[]={
 
 static const intel_x86_umask_t skl_l2_lines_out[]={
   { .uname = "NON_SILENT",
-    .udesc  = "TBD",
+    .udesc  = "Counts the number of lines that are evicted by L2 cache when triggered by an L2 cache fill. Those lines can be either in modified state or clean state. Modified lines may either be written back to L3 or directly written to memory and not allocated in L3.  Clean lines may either be allocated in L3 or dropped ",
     .ucode  = 0x200,
     .uflags = INTEL_X86_NCOMBO,
   },
-  { .uname = "USELESS_PREF",
-    .udesc  = "TBD",
+  { .uname = "USELESS_HWPREF",
+    .udesc  = "Counts the number of lines that have been hardware prefetched but not used and now evicted by L2 cache",
+    .ucode  = 0x400,
+    .uequiv = "USELESS_HWPF",
+    .uflags = INTEL_X86_NCOMBO,
+  },
+  { .uname = "USELESS_HWPF",
+    .udesc  = "Counts the number of lines that have been hardware prefetched but not used and now evicted by L2 cache",
     .ucode  = 0x400,
     .uflags = INTEL_X86_NCOMBO,
   },
   { .uname = "SILENT",
-    .udesc  = "TBD",
+    .udesc  = "Counts the number of lines that are silently dropped by L2 cache when triggered by an L2 cache fill. These lines are typically in Shared state. This is a per-core event.",
     .ucode  = 0x100,
     .uflags = INTEL_X86_NCOMBO,
   },
@@ -789,6 +844,32 @@ static const intel_x86_umask_t skl_mem_load_l3_hit_retired[]={
   },
 };
 
+static const intel_x86_umask_t skl_mem_load_l3_miss_retired[]={
+  { .uname = "LOCAL_DRAM",
+    .udesc  = "Retired load instructions which data sources missed L3 but serviced from local dram",
+    .ucode  = 0x100,
+    .uflags = INTEL_X86_NCOMBO | INTEL_X86_PEBS,
+  },
+  { .uname = "REMOTE_DRAM",
+    .udesc  = "Retired load instructions which data sources missed L3 but serviced from remote dram",
+    .ucode  = 0x200,
+    .umodel = PFM_PMU_INTEL_SKX,
+    .uflags = INTEL_X86_NCOMBO | INTEL_X86_PEBS,
+  },
+  { .uname = "REMOTE_HITM",
+    .udesc  = "Retired load instructions whose data sources was remote HITM",
+    .ucode  = 0x400,
+    .umodel = PFM_PMU_INTEL_SKX,
+    .uflags = INTEL_X86_NCOMBO | INTEL_X86_PEBS,
+  },
+  { .uname = "REMOTE_FWD",
+    .udesc  = "Retired load instructions whose data sources was remote HITM",
+    .ucode  = 0x800,
+    .umodel = PFM_PMU_INTEL_SKX,
+    .uflags = INTEL_X86_NCOMBO | INTEL_X86_PEBS,
+  },
+};
+
 static const intel_x86_umask_t skl_mem_load_retired[]={
   { .uname = "L1_HIT",
     .udesc  = "Retired load uops with L1 cache hits as data source",
@@ -1105,7 +1186,7 @@ static const intel_x86_umask_t skl_uops_executed[]={
   },
 };
 
-static const intel_x86_umask_t skl_uops_dispatched[]={
+static const intel_x86_umask_t skl_uops_dispatched_port[]={
   { .uname = "PORT_0",
     .udesc  = "Cycles which a Uop is executed on port 0",
     .ucode  = 0x100,
@@ -1312,54 +1393,40 @@ static const intel_x86_umask_t skl_offcore_response[]={
      .ucode = 1ULL << (1 + 8),
      .grpid = 0,
    },
-   { .uname  = "DMND_IFETCH",
+   { .uname  = "DMND_CODE_RD",
      .udesc  = "Request: number of demand and DCU prefetch instruction cacheline reads. Does not count L2 code read prefetches",
      .ucode = 1ULL << (2 + 8),
      .grpid = 0,
    },
-   { .uname  = "WB",
-     .udesc  = "Request: number of writebacks (modified to exclusive) transactions",
-     .ucode = 1ULL << (3 + 8),
-     .grpid = 0,
+
+   { .uname  = "PF_L2_DATA_RD",
+     .udesc  = "Request: number of data prefetch requests to L2",
+     .ucode  = 1ULL << (4 + 8),
+     .umodel = PFM_PMU_INTEL_SKX,
+     .grpid  = 0,
    },
-   { .uname  = "PF_DATA_RD",
-     .udesc  = "Request: number of data cacheline reads generated by L2 prefetchers",
-     .ucode = 1ULL << (4 + 8),
-     .grpid = 0,
-   },
-   { .uname  = "PF_RFO",
-     .udesc  = "Request: number of RFO requests generated by L2 prefetchers",
+   { .uname  = "PF_L2_RFO",
+     .udesc  = "Request: number of RFO prefetch requests to L2",
      .ucode = 1ULL << (5 + 8),
-     .grpid = 0,
-   },
-   { .uname  = "PF_IFETCH",
-     .udesc  = "Request: number of code reads generated by L2 prefetchers",
-     .ucode = 1ULL << (6 + 8),
+     .umodel = PFM_PMU_INTEL_SKX,
      .grpid = 0,
    },
    { .uname  = "PF_L3_DATA_RD",
-     .udesc  = "Request: number of L2 prefetcher requests for loads that end up in L3",
+     .udesc  = "Request: number of data prefetch requests for loads that end up in L3",
      .ucode = 1ULL << (7 + 8),
+     .umodel = PFM_PMU_INTEL_SKX,
      .grpid = 0,
    },
    { .uname  = "PF_L3_RFO",
-     .udesc  = "Request: number of RFO requests generated by L2 prefetcher that end up in L3",
+     .udesc  = "Request: number of RFO prefetch requests that end up in L3",
      .ucode = 1ULL << (8 + 8),
+     .umodel = PFM_PMU_INTEL_SKX,
      .grpid = 0,
    },
-   { .uname  = "PF_L3_IFETCH",
-     .udesc  = "Request: number of L2 prefetcher requests to L3 for instruction fetches",
-     .ucode = 1ULL << (9 + 8),
-     .grpid = 0,
-   },
-   { .uname  = "BUS_LOCKS",
-     .udesc  = "Request: number bus lock and split lock requests",
+   { .uname  = "PF_L1D_AND_SW",
+     .udesc  = "Request: number of L1 data cache hardware prefetch requests and software prefetch requests",
      .ucode = 1ULL << (10 + 8),
-     .grpid = 0,
-   },
-   { .uname  = "STRM_ST",
-     .udesc  = "Request: number of streaming store requests",
-     .ucode = 1ULL << (11 + 8),
+     .umodel = PFM_PMU_INTEL_SKX,
      .grpid = 0,
    },
    { .uname  = "OTHER",
@@ -1367,81 +1434,119 @@ static const intel_x86_umask_t skl_offcore_response[]={
      .ucode = 1ULL << (15+8),
      .grpid = 0,
    },
-   { .uname  = "ANY_IFETCH",
-     .udesc  = "Request: combination of PF_IFETCH | DMND_IFETCH | PF_L3_IFETCH",
-     .uequiv = "PF_IFETCH:DMND_IFETCH:PF_L3_IFETCH",
-     .ucode = 0x24100,
+
+   { .uname  = "ANY_REQUEST",
+     .udesc  = "Request: combination of all request umasks",
+     .uequiv = "DMND_DATA_RD:DMND_RFO:DMND_CODE_RD:OTHER",
+     .ucode = 0x1800700,
+     .uflags= INTEL_X86_NCOMBO | INTEL_X86_DFL,
+     .umodel = PFM_PMU_INTEL_SKL,
      .grpid = 0,
    },
    { .uname  = "ANY_REQUEST",
      .udesc  = "Request: combination of all request umasks",
-     .uequiv = "DMND_DATA_RD:DMND_RFO:DMND_IFETCH:WB:PF_DATA_RD:PF_RFO:PF_IFETCH:PF_L3_DATA_RD:PF_L3_RFO:PF_L3_IFETCH:BUS_LOCKS:STRM_ST:OTHER",
-     .ucode = 0x8fff00,
+     .uequiv = "DMND_DATA_RD:DMND_RFO:DMND_CODE_RD:PF_L2_DATA_RD:PF_L2_RFO:PF_L3_DATA_RD:PF_L3_RFO:PF_L1D_AND_SW:OTHER",
+     .ucode = 0x85b700,
      .uflags= INTEL_X86_NCOMBO | INTEL_X86_DFL,
+     .umodel = PFM_PMU_INTEL_SKX,
+     .grpid = 0,
+   },
+   { .uname  = "ANY_DATA_RD",
+     .udesc  = "Request: combination of DMND_DATA_RD | PF_L2_DATA_RD | PF_L3_DATA_RD | PF_L1D_AND_SW",
+     .uequiv = "DMND_DATA_RD:PF_L2_DATA_RD:PF_L3_DATA_RD:PF_L1D_AND_SW",
+     .ucode = 0x1049100,
+     .umodel = PFM_PMU_INTEL_SKX,
      .grpid = 0,
    },
    { .uname  = "ANY_DATA",
-     .udesc  = "Request: combination of DMND_DATA | PF_DATA_RD | PF_LLC_DATA_RD",
-     .uequiv = "DMND_DATA_RD:PF_DATA_RD:PF_L3_DATA_RD",
-     .ucode = 0x9100,
+     .udesc  = "Request: combination of ANY_DATA_RD | PF_L2_RFO | PF_L3_RFO | DMND_RFO",
+     .uequiv = "ANY_DATA_RD:DMND_RFO:PF_L2_RFO:PF_L3_RFO",
+     .ucode = 0x105b300,
+     .umodel = PFM_PMU_INTEL_SKX,
+     .grpid = 0,
+   },
+   { .uname  = "ANY_DATA_PF",
+     .udesc  = "Request: combination of PF_L2_DATA_RD | PF_L3_DATA_RD | PF_L1D_AND_SW",
+     .uequiv = "PF_L2_DATA_RD:PF_L3_DATA_RD:PF_L1D_AND_SW",
+     .ucode = 0x1049000,
+     .umodel = PFM_PMU_INTEL_SKX,
      .grpid = 0,
    },
    { .uname  = "ANY_RFO",
-     .udesc  = "Request: combination of DMND_RFO | PF_RFO | PF_LLC_RFO",
-     .uequiv = "DMND_RFO:PF_RFO:PF_L3_RFO",
-     .ucode = 0x10300,
+     .udesc  = "Request: combination of DMND_RFO | PF_L2_RFO | PF_L3_RFO",
+     .uequiv = "DMND_RFO:PF_L2_RFO:PF_L3_RFO",
+     .ucode = 0x1012200,
+     .umodel = PFM_PMU_INTEL_SKX,
      .grpid = 0,
    },
 
    { .uname  = "ANY_RESPONSE",
      .udesc  = "Response: count any response type",
-     .ucode = 1ULL << (16+8),
-     .uflags= INTEL_X86_NCOMBO | INTEL_X86_DFL | INTEL_X86_EXCL_GRP_GT,
-     .grpid = 1,
+     .ucode  = 1ULL << (16+8),
+     .uflags = INTEL_X86_NCOMBO | INTEL_X86_DFL | INTEL_X86_EXCL_GRP_GT,
+     .grpid  = 1,
+   },
+   { .uname  = "SUPPLIER_NONE",
+     .udesc  = "Supplier: counts number of times supplier information is not available",
+     .ucode  = 1ULL << (17+8),
+     .grpid  = 1,
    },
    { .uname  = "NO_SUPP",
      .udesc  = "Supplier: counts number of times supplier information is not available",
-     .ucode = 1ULL << (17+8),
-     .grpid = 1,
+     .ucode  = 1ULL << (17+8),
+     .uequiv = "SUPPLIER_NONE",
+     .grpid  = 1,
    },
    { .uname  = "L3_HITM",
      .udesc  = "Supplier: counts L3 hits in M-state (initial lookup)",
-     .ucode = 1ULL << (18+8),
-     .umodel = PFM_PMU_INTEL_SKL,
-     .grpid = 1,
+     .ucode  = 1ULL << (18+8),
+     .grpid  = 1,
    },
    { .uname  = "L3_HITE",
      .udesc  = "Supplier: counts L3 hits in E-state",
-     .ucode = 1ULL << (19+8),
-     .umodel = PFM_PMU_INTEL_SKL,
-     .grpid = 1,
+     .ucode  = 1ULL << (19+8),
+     .grpid  = 1,
    },
    { .uname  = "L3_HITS",
      .udesc  = "Supplier: counts L3 hits in S-state",
-     .ucode = 1ULL << (20+8),
-     .umodel = PFM_PMU_INTEL_SKL,
-     .grpid = 1,
+     .ucode  = 1ULL << (20+8),
+     .grpid  = 1,
    },
    { .uname  = "L3_HITF",
      .udesc  = "Supplier: counts L3 hits in F-state",
-     .ucode = 1ULL << (21+8),
+     .ucode  = 1ULL << (21+8),
+     .umodel = PFM_PMU_INTEL_SKX,
+     .grpid  = 1,
+   },
+   { .uname  = "L3_HITMES",
+     .udesc  = "Supplier: counts L3 hits in any state (M, E, S)",
+     .ucode  = 0x3ULL << (18+8),
+     .uequiv = "L3_HITM:L3_HITE:L3_HITS",
      .umodel = PFM_PMU_INTEL_SKL,
-     .grpid = 1,
+     .grpid  = 1,
+   },
+   { .uname  = "L3_HIT",
+     .udesc  = "Alias for L3_HITMES",
+     .ucode  = 0x3ULL << (18+8),
+     .uequiv = "L3_HITMES",
+     .umodel = PFM_PMU_INTEL_SKL,
+     .grpid  = 1,
    },
    { .uname  = "L3_HITMESF",
      .udesc  = "Supplier: counts L3 hits in any state (M, E, S, F)",
      .ucode  = 0xfULL << (18+8),
      .uequiv = "L3_HITM:L3_HITE:L3_HITS:L3_HITF",
-     .umodel = PFM_PMU_INTEL_SKL,
+     .umodel = PFM_PMU_INTEL_SKX,
      .grpid  = 1,
    },
    { .uname  = "L3_HIT",
-     .udesc  = "Alias for L3_HITMESF",
-     .ucode  = 0xfULL << (18+8),
-     .uequiv = "L3_HITM:L3_HITE:L3_HITS:L3_HITF",
-     .umodel = PFM_PMU_INTEL_SKL,
+     .udesc  = "Alias for L3_HITMES",
+     .ucode  = 0x3ULL << (18+8),
+     .uequiv = "L3_HITMESF",
+     .umodel = PFM_PMU_INTEL_SKX,
      .grpid  = 1,
    },
+
    { .uname  = "L4_HIT_LOCAL_L4",
      .udesc  = "Supplier: L4 local hit",
      .ucode  = 0x1ULL << (22+8),
@@ -1450,57 +1555,66 @@ static const intel_x86_umask_t skl_offcore_response[]={
    },
    { .uname  = "L3_MISS_LOCAL",
      .udesc  = "Supplier: counts L3 misses to local DRAM",
-     .ucode = 1ULL << (26+8),
-     .umodel = PFM_PMU_INTEL_SKL,
-     .grpid = 1,
+     .ucode  = 1ULL << (26+8),
+     .grpid  = 1,
+   },
+   { .uname  = "L3_MISS_MISS_REMOTE_HOP1_DRAM",
+     .udesc  = "Supplier: counts L3 misses to remote DRAM with 1 hop",
+     .ucode  = 1ULL << (28+8),
+     .grpid  = 1,
    },
    { .uname  = "L3_MISS",
      .udesc  = "Supplier: counts L3 misses",
-     .ucode = 0xfULL << (26+8),
-     .grpid = 1,
+     .ucode  = 0x1ULL << (26+8),
+     .uequiv = "L3_MISS_LOCAL",
+     .umodel = PFM_PMU_INTEL_SKL,
+     .grpid  = 1,
    },
+   { .uname  = "L3_MISS",
+     .udesc  = "Supplier: counts L3 misses (local or remote)",
+     .ucode  = 0xfULL << (26+8),
+     .uequiv = "L3_MISS_LOCAL",
+     .umodel = PFM_PMU_INTEL_SKX,
+     .grpid  = 1,
+   },
+
    { .uname  = "SPL_HIT",
      .udesc  = "Snoop: counts L3 supplier hit",
-     .ucode = 0x1ULL << (30+8),
-     .grpid = 1,
+     .ucode  = 0x1ULL << (30+8),
+     .umodel = PFM_PMU_INTEL_SKL,
+     .grpid  = 1,
    },
    { .uname  = "SNP_NONE",
      .udesc  = "Snoop: counts number of times no snoop-related information is available",
-     .ucode = 1ULL << (31+8),
-     .grpid = 2,
+     .ucode  = 1ULL << (31+8),
+     .grpid  = 2,
    },
    { .uname  = "SNP_NOT_NEEDED",
      .udesc  = "Snoop: counts the number of times no snoop was needed to satisfy the request",
-     .ucode = 1ULL << (32+8),
-     .grpid = 2,
+     .ucode  = 1ULL << (32+8),
+     .grpid  = 2,
    },
    { .uname  = "SNP_MISS",
      .udesc  = "Snoop: counts number of times a snoop was needed and it missed all snooped caches",
-     .ucode = 1ULL << (33+8),
-     .grpid = 2,
+     .ucode  = 1ULL << (33+8),
+     .grpid  = 2,
    },
-   { .uname  = "SNP_NO_FWD",
+   { .uname  = "SNP_HIT_NO_FWD",
      .udesc  = "Snoop: counts number of times a snoop was needed and it hit in at leas one snooped cache",
-     .ucode = 1ULL << (34+8),
-     .grpid = 2,
+     .ucode  = 1ULL << (34+8),
+     .grpid  = 2,
    },
-   { .uname  = "SNP_FWD",
+   { .uname  = "SNP_HIT_WITH_FWD",
      .udesc  = "Snoop: counts number of times a snoop was needed and data was forwarded from a remote socket",
-     .ucode = 1ULL << (35+8),
-     .grpid = 2,
-   },
-   { .uname  = "HITM",
-     .udesc  = "Snoop: counts number of times a snoop was needed and it hitM-ed in local or remote cache",
-     .ucode = 1ULL << (36+8),
-     .uequiv = "SNP_HITM",
-     .grpid = 2,
+     .ucode  = 1ULL << (35+8),
+     .grpid  = 2,
    },
    { .uname  = "SNP_HITM",
      .udesc  = "Snoop: counts number of times a snoop was needed and it hitM-ed in local or remote cache",
      .ucode = 1ULL << (36+8),
      .grpid = 2,
    },
-   { .uname  = "NON_DRAM",
+   { .uname  = "SNP_NON_DRAM",
      .udesc  = "Snoop:  counts number of times target was a non-DRAM system address. This includes MMIO transactions",
      .ucode = 1ULL << (37+8),
      .grpid = 2,
@@ -1508,7 +1622,7 @@ static const intel_x86_umask_t skl_offcore_response[]={
    { .uname  = "SNP_ANY",
      .udesc  = "Snoop: any snoop reason",
      .ucode = 0x7fULL << (31+8),
-     .uequiv = "SNP_NONE:SNP_NOT_NEEDED:SNP_MISS:SNP_NO_FWD:SNP_FWD:HITM:NON_DRAM",
+     .uequiv = "SNP_NONE:SNP_NOT_NEEDED:SNP_MISS:SNP_HIT_NO_FWD:SNP_HIT_WITH_FWD:SNP_HITM:SNP_NON_DRAM",
      .uflags= INTEL_X86_DFL,
      .grpid = 2,
    },
@@ -1859,7 +1973,7 @@ static const intel_x86_umask_t skl_fp_arith[]={
 };
 
 static const intel_x86_umask_t skl_exe_activity[]={
-   { .uname  = "",
+   { .uname  = "1_PORTS_UTIL",
      .udesc  = "Cycles with 1 uop executing across all ports and Reservation Station is not empty",
      .ucode = 0x0200,
      .uflags= INTEL_X86_NCOMBO,
@@ -1955,6 +2069,77 @@ static const intel_x86_umask_t skl_offcore_requests_buffer[]={
    },
 };
 
+static const intel_x86_umask_t skl_mem_load_misc_retired[]={
+   { .uname  = "UC",
+     .udesc  = "Number of uncached load retired",
+     .ucode = 0x400,
+     .uflags= INTEL_X86_PEBS | INTEL_X86_DFL,
+   },
+};
+
+static const intel_x86_umask_t skl_idi_misc[]={
+   { .uname  = "WB_UPGRADE",
+     .udesc  = "Counts number of cache lines that are allocated and written back to L3 with the intention that they are more likely to be reused shortly",
+     .ucode = 0x200,
+     .uflags= INTEL_X86_NCOMBO,
+   },
+   { .uname  = "WB_DOWNGRADE",
+     .udesc  = "Counts number of cache lines that are dropped and not written back to L3 as they are deemed to be less likely to be reused shortly",
+     .ucode = 0x400,
+     .uflags= INTEL_X86_NCOMBO,
+   },
+};
+
+static const intel_x86_umask_t skl_core_power[]={
+   { .uname  = "LVL0_TURBO_LICENSE",
+     .udesc  = "Number of core cycles where the core was running in a manner where Turbo may be clipped to the Non-AVX turbo schedule.",
+     .ucode = 0x700,
+     .umodel = PFM_PMU_INTEL_SKX,
+     .uflags= INTEL_X86_NCOMBO,
+   },
+   { .uname  = "LVL1_TURBO_LICENSE",
+     .udesc  = "Number of core cycles where the core was running in a manner where Turbo may be clipped to the AVX2 turbo schedule.",
+     .ucode = 0x1800,
+     .umodel = PFM_PMU_INTEL_SKX,
+     .uflags= INTEL_X86_NCOMBO,
+   },
+   { .uname  = "LVL2_TURBO_LICENSE",
+     .udesc  = "Number of core cycles where the core was running in a manner where Turbo may be clipped to the AVX512 turbo schedule.",
+     .ucode = 0x2000,
+     .umodel = PFM_PMU_INTEL_SKX,
+     .uflags= INTEL_X86_NCOMBO,
+   },
+   { .uname  = "THROTTLE",
+     .udesc  = "Number of core cycles where the core was throttled due to a pending power level request.",
+     .ucode = 0x4000,
+     .umodel = PFM_PMU_INTEL_SKX,
+     .uflags= INTEL_X86_NCOMBO,
+   },
+};
+
+static const intel_x86_umask_t skl_sw_prefetch[]={
+   { .uname  = "NTA",
+     .udesc  = "Number of prefetch.nta instructions executed",
+     .ucode = 0x100,
+     .uflags= INTEL_X86_NCOMBO,
+   },
+   { .uname  = "T0",
+     .udesc  = "Number of prefetch.t0 instructions executed",
+     .ucode = 0x200,
+     .uflags= INTEL_X86_NCOMBO,
+   },
+   { .uname  = "T1_T2",
+     .udesc  = "Number prefetch.t1 or prefetch.t2 instructions executed",
+     .ucode = 0x4000,
+     .uflags= INTEL_X86_NCOMBO,
+   },
+   { .uname  = "PREFETCHW",
+     .udesc  = "Number prefetch.w instructions executed",
+     .ucode = 0x8000,
+     .uflags= INTEL_X86_NCOMBO,
+   },
+};
+
 static const intel_x86_entry_t intel_skl_pe[]={
   { .name   = "UNHALTED_CORE_CYCLES",
     .desc   = "Count core clock cycles whenever the clock signal on the specific core is running (not halted)",
@@ -2289,6 +2474,16 @@ static const intel_x86_entry_t intel_skl_pe[]={
     .numasks = LIBPFM_ARRAY_SIZE(skl_mem_load_l3_hit_retired),
     .umasks  = skl_mem_load_l3_hit_retired
   },
+  { .name = "MEM_LOAD_UOPS_L3_MISS_RETIRED",
+    .desc   = "L3 miss load uops retired (Precise Event)",
+    .code = 0xd3,
+    .cntmsk = 0xf,
+    .ngrp = 1,
+    .flags = INTEL_X86_PEBS,
+    .modmsk = INTEL_V4_ATTRS,
+    .numasks = LIBPFM_ARRAY_SIZE(skl_mem_load_l3_miss_retired),
+    .umasks  = skl_mem_load_l3_miss_retired
+  },
   { .name = "MEM_LOAD_UOPS_LLC_HIT_RETIRED",
     .desc   = "L3 hit load uops retired (Precise Event)",
     .equiv = "MEM_LOAD_L3_HIT_RETIRED",
@@ -2452,15 +2647,24 @@ static const intel_x86_entry_t intel_skl_pe[]={
     .numasks = LIBPFM_ARRAY_SIZE(skl_lsd),
     .umasks  = skl_lsd,
   },
-
-  { .name = "UOPS_DISPATCHED",
-    .desc   = "Uops dispatch to specific ports",
+  { .name = "UOPS_DISPATCHED_PORT",
+    .desc   = "Uops dispatched to specific ports",
     .code = 0xa1,
     .cntmsk = 0xff,
     .ngrp = 1,
     .modmsk = INTEL_V4_ATTRS,
-    .numasks = LIBPFM_ARRAY_SIZE(skl_uops_dispatched),
-    .umasks  = skl_uops_dispatched,
+    .numasks = LIBPFM_ARRAY_SIZE(skl_uops_dispatched_port),
+    .umasks  = skl_uops_dispatched_port,
+  },
+  { .name = "UOPS_DISPATCHED",
+    .desc   = "Uops dispatched to specific ports",
+    .equiv = "UOPS_DISPATCHED_PORT",
+    .code = 0xa1,
+    .cntmsk = 0xff,
+    .ngrp = 1,
+    .modmsk = INTEL_V4_ATTRS,
+    .numasks = LIBPFM_ARRAY_SIZE(skl_uops_dispatched_port),
+    .umasks  = skl_uops_dispatched_port,
   },
   { .name = "UOPS_ISSUED",
     .desc   = "Uops issued",
@@ -2581,6 +2785,64 @@ static const intel_x86_entry_t intel_skl_pe[]={
     .ngrp = 1,
     .umasks = skl_hw_interrupts,
   },
+  { .name   = "SQ_MISC",
+    .desc   = "SuperQueue miscellaneous",
+    .modmsk = INTEL_V4_ATTRS,
+    .cntmsk = 0xf,
+    .code = 0xf4,
+    .numasks = LIBPFM_ARRAY_SIZE(skl_sq_misc),
+    .ngrp = 1,
+    .umasks = skl_sq_misc,
+  },
+  { .name   = "MEM_LOAD_MISC_RETIRED",
+    .desc   = "Load retired miscellaneous",
+    .modmsk = INTEL_V4_ATTRS,
+    .flags  = INTEL_X86_PEBS,
+    .cntmsk = 0xf,
+    .code = 0xd4,
+    .numasks = LIBPFM_ARRAY_SIZE(skl_mem_load_misc_retired),
+    .ngrp = 1,
+    .umasks = skl_mem_load_misc_retired,
+  },
+  { .name   = "IDI_MISC",
+    .desc   = "Miscellaneous",
+    .modmsk = INTEL_V4_ATTRS,
+    .cntmsk = 0xf,
+    .code = 0xfe,
+    .numasks = LIBPFM_ARRAY_SIZE(skl_idi_misc),
+    .model = PFM_PMU_INTEL_SKX,
+    .ngrp = 1,
+    .umasks = skl_idi_misc,
+  },
+  { .name   = "CORE_POWER",
+    .desc   = "Power power cycles",
+    .modmsk = INTEL_V4_ATTRS,
+    .cntmsk = 0xf,
+    .code = 0x28,
+    .numasks = LIBPFM_ARRAY_SIZE(skl_core_power),
+    .model  = PFM_PMU_INTEL_SKX,
+    .ngrp = 1,
+    .umasks = skl_core_power,
+  },
+  { .name   = "SW_PREFETCH",
+    .desc   = "Software prefetches",
+    .modmsk = INTEL_V4_ATTRS,
+    .equiv  = "SW_PREFETCH_ACCESS",
+    .cntmsk = 0xf,
+    .code = 0x32,
+    .numasks = LIBPFM_ARRAY_SIZE(skl_sw_prefetch),
+    .ngrp = 1,
+    .umasks = skl_sw_prefetch,
+  },
+  { .name   = "SW_PREFETCH_ACCESS",
+    .desc   = "Software prefetches",
+    .modmsk = INTEL_V4_ATTRS,
+    .cntmsk = 0xf,
+    .code = 0x32,
+    .numasks = LIBPFM_ARRAY_SIZE(skl_sw_prefetch),
+    .ngrp = 1,
+    .umasks = skl_sw_prefetch,
+  },
   { .name   = "OFFCORE_REQUESTS_BUFFER",
     .desc   = "Offcore requests buffer",
     .modmsk = INTEL_V4_ATTRS,
