@@ -109,6 +109,47 @@ pfm_intel_hswep_unc_detect(void *this)
        return PFM_SUCCESS;
 }
 
+int
+pfm_intel_knl_unc_detect(void *this)
+{
+       int ret;
+
+       ret = pfm_intel_x86_detect();
+       if (ret != PFM_SUCCESS)
+
+       if (pfm_intel_x86_cfg.family != 6)
+               return PFM_ERR_NOTSUPP;
+
+       switch(pfm_intel_x86_cfg.model) {
+               case 87: /* Knights Landing */
+                         break;
+               default:
+                       return PFM_ERR_NOTSUPP;
+       }
+       return PFM_SUCCESS;
+}
+
+int
+pfm_intel_bdx_unc_detect(void *this)
+{
+       int ret;
+
+       ret = pfm_intel_x86_detect();
+       if (ret != PFM_SUCCESS)
+
+       if (pfm_intel_x86_cfg.family != 6)
+               return PFM_ERR_NOTSUPP;
+
+       switch(pfm_intel_x86_cfg.model) {
+               case 79: /* Broadwell X */
+               case 86: /* Broadwell X */
+                         break;
+               default:
+                       return PFM_ERR_NOTSUPP;
+       }
+       return PFM_SUCCESS;
+}
+
 static void
 display_com(void *this, pfmlib_event_desc_t *e, void *val)
 {
@@ -158,7 +199,7 @@ snbep_unc_add_defaults(void *this, pfmlib_event_desc_t *e,
 			   unsigned int msk,
 			   uint64_t *umask,
 			   pfm_snbep_unc_reg_t *filters,
-			   unsigned int max_grpid)
+			   unsigned short max_grpid)
 {
 	const intel_x86_entry_t *pe = this_pe(this);
 	const intel_x86_entry_t *ent;
@@ -233,7 +274,7 @@ snbep_unc_add_defaults(void *this, pfmlib_event_desc_t *e,
 			}
 		}
 		if (!added && !skip) {
-			DPRINT("no default found for event %s unit mask group %d (max_grpid=%d)\n", ent->name, i, max_grpid);
+			DPRINT("no default found for event %s unit mask group %d (max_grpid=%d, i=%d)\n", ent->name, i, max_grpid, i);
 			return PFM_ERR_UMASK;
 		}
 	}
@@ -252,18 +293,18 @@ pfm_intel_snbep_unc_get_encoding(void *this, pfmlib_event_desc_t *e)
 {
 	const intel_x86_entry_t *pe = this_pe(this);
 	unsigned int grpmsk, ugrpmsk = 0;
-	unsigned int max_grpid = INTEL_X86_MAX_GRPID;
-	unsigned int last_grpid =  INTEL_X86_MAX_GRPID;
+	unsigned short max_grpid = INTEL_X86_MAX_GRPID;
+	unsigned short last_grpid =  INTEL_X86_MAX_GRPID;
 	int umodmsk = 0, modmsk_r = 0;
 	int pcu_filt_band = -1;
 	pfm_snbep_unc_reg_t reg;
 	pfm_snbep_unc_reg_t filters[INTEL_X86_MAX_FILTERS];
 	pfm_snbep_unc_reg_t addr;
-	pfm_event_attr_info_t *a;
+	pfmlib_event_attr_info_t *a;
 	uint64_t val, umask1, umask2;
 	int k, ret;
 	int has_cbo_tid = 0;
-	unsigned int grpid;
+	unsigned short grpid;
 	int grpcounts[INTEL_X86_NUM_GRP];
 	int ncombo[INTEL_X86_NUM_GRP];
 	char umask_str[PFMLIB_EVT_MAX_NAME_LEN];
@@ -619,7 +660,7 @@ pfm_intel_snbep_unc_can_auto_encode(void *this, int pidx, int uidx)
 }
 
 int
-pfm_intel_snbep_unc_get_event_attr_info(void *this, int pidx, int attr_idx, pfm_event_attr_info_t *info)
+pfm_intel_snbep_unc_get_event_attr_info(void *this, int pidx, int attr_idx, pfmlib_event_attr_info_t *info)
 {
 	const intel_x86_entry_t *pe = this_pe(this);
 	const pfmlib_attr_desc_t *atdesc = this_atdesc(this);
